@@ -11,7 +11,6 @@
 from __future__ import absolute_import, division, print_function
 
 import argparse
-import json
 import multiprocessing.pool
 import os
 import platform
@@ -24,8 +23,8 @@ import sys
 import tarfile
 import threading
 import time
-import warnings
 import zipfile
+import multiprocessing
 
 try:  # Python 3
     from urllib.error import HTTPError, URLError
@@ -50,7 +49,7 @@ devnull = open(os.devnull, "wb")  # to redirect unwanted subprocess output
 allowed_ssh_connections = {}
 concurrent_git_connection_limit = threading.Semaphore(5)
 
-_prebuilt_cctbx_base = "v2023.9"
+_prebuilt_cctbx_base = "2024.9"
 
 
 def make_executable(filepath):
@@ -61,7 +60,36 @@ def make_executable(filepath):
         os.chmod(filepath, mode)
 
 
-def install_micromamba(python, include_cctbx, cmake):
+def _run_conda_retry(command_list):
+    for retry in range(5):
+        retry += 1
+        try:
+            run_command(
+                command=command_list,
+                workdir=".",
+            )
+        except Exception:
+            print(
+                """
+*******************************************************************************
+There was a failure in constructing the conda environment.
+Attempt {retry} of 5 will start {retry} minute(s) from {t}.
+*******************************************************************************
+""".format(retry=retry, t=time.asctime())
+            )
+            time.sleep(retry * 60)
+        else:
+            break
+    else:
+        sys.exit(
+            """
+The conda environment could not be constructed. Please check that there is a
+working network connection for downloading conda packages.
+"""
+        )
+
+
+def install_micromamba(python, cmake):
     """Download and install Micromamba"""
     if sys.platform.startswith("linux"):
         conda_platform = "linux"
@@ -82,7 +110,7 @@ def install_micromamba(python, include_cctbx, cmake):
         raise NotImplementedError(
             "Unsupported platform %s / %s" % (os.name, sys.platform)
         )
-    url = "https://micro.mamba.pm/api/micromamba/{0}/latest".format(conda_arch)
+    url = "https://micromamba.snakepit.net/api/micromamba/{0}/latest".format(conda_arch)
     mamba_prefix = os.path.realpath("micromamba")
     clean_env["MAMBA_ROOT_PREFIX"] = mamba_prefix
     mamba = os.path.join(mamba_prefix, member.split("/")[-1])
@@ -145,252 +173,52 @@ def install_micromamba(python, include_cctbx, cmake):
         "mamba",
         python_requirement,
     ]
-    if include_cctbx or cmake:
-        command_list.append("cctbx-nightly::cctbx-base=" + _prebuilt_cctbx_base)
+    extra_deps = []
     if cmake:
-        command_list.extend(["pycbf", "cmake"])
-    if os.name == "nt":
-        # Installing pre-commit via precommittbx does not work on windows
-        command_list.append("pre-commit")
+        extra_deps = [
+            "cctbx-base=" + _prebuilt_cctbx_base,
+            "pycbf",
+            "cmake",
+            "pre-commit",
+        ]
+        if _prebuilt_cctbx_base == "2024.7":
+            # Known minor incompatibility causes this to otherwise be removed
+            extra_deps.append("libboost-python-devel")
+        # If we're running from an explicit requirements file, then we
+        # need to install extra dependencies in a separate stage
+        with open(filename) as dep_file:
+            is_explicit_reqs = "@EXPLICIT" in dep_file.read()
+
+        if not is_explicit_reqs:
+            command_list.extend(extra_deps)
+            extra_deps = []
 
     print(
         "{text} dials environment from {filename} with Python {python}".format(
             text=text_messages[0], filename=filename, python=python
         )
     )
-    for retry in range(5):
-        retry += 1
-        try:
-            run_command(
-                command=command_list,
-                workdir=".",
-            )
-        except Exception:
-            print(
-                """
-*******************************************************************************
-There was a failure in constructing the conda environment.
-Attempt {retry} of 5 will start {retry} minute(s) from {t}.
-*******************************************************************************
-""".format(
-                    retry=retry, t=time.asctime()
-                )
-            )
-            time.sleep(retry * 60)
-        else:
-            break
-    else:
-        sys.exit(
-            """
-The conda environment could not be constructed. Please check that there is a
-working network connection for downloading conda packages.
-"""
-        )
-    print("Completed {text}:\n  {prefix}".format(text=text_messages[1], prefix=prefix))
-    with open(os.path.join(prefix, ".condarc"), "w") as fh:
-        fh.write(
-            """
-changeps1: False
-channels:
-  - conda-forge
-""".lstrip()
-        )
 
-
-def install_miniconda(location):
-    """Download and install Miniconda3"""
-    if sys.platform.startswith("linux"):
-        filename = "Miniconda3-latest-Linux-x86_64.sh"
-    elif sys.platform == "darwin":
-        filename = "Miniconda3-latest-MacOSX-x86_64.sh"
-    elif os.name == "nt":
-        filename = "Miniconda3-latest-Windows-x86_64.exe"
-    else:
-        raise NotImplementedError(
-            "Unsupported platform %s / %s" % (os.name, sys.platform)
-        )
-    url = "https://repo.anaconda.com/miniconda/" + filename
-    filename = os.path.join(location, filename)
-
-    print("Downloading {url}:".format(url=url), end=" ")
-    result = download_to_file(url, filename)
-    if result in (0, -1):
-        sys.exit("Miniconda download failed")
-
-    # run the installer
-    if os.name == "nt":
-        command = [
-            filename,
-            "/InstallationType=JustMe",
-            "/RegisterPython=0",
-            "/AddToPath=0",
-            "/S",
-            "/D=" + location,
-        ]
-    else:
-        command = ["/bin/sh", filename, "-b", "-u", "-p", location]
-
-    print("Installing Miniconda")
-    run_command(command=command, workdir=".")
-
-
-def install_conda(python, include_cctbx, cmake):
-    # Find relevant conda base installation
-    conda_base = os.path.realpath("miniconda")
-    if os.name == "nt":
-        conda_exe = os.path.join(conda_base, "Scripts", "conda.exe")
-    else:
-        conda_exe = os.path.join(conda_base, "bin", "conda")
-
-    # default environment file for users
-    environment_file = os.path.join(
-        os.path.expanduser("~"), ".conda", "environments.txt"
-    )
-
-    def get_environments():
-        """Return a set of existing conda environment paths"""
-        try:
-            with open(environment_file) as f:
-                paths = f.readlines()
-        except IOError:
-            paths = []
-        environments = set(  # noqa; C401, Python 2.7 compatibility
-            os.path.normpath(env.strip()) for env in paths if os.path.isdir(env.strip())
-        )
-        env_dirs = (
-            os.path.join(conda_base, "envs"),
-            os.path.join(os.path.expanduser("~"), ".conda", "envs"),
-        )
-        for env_dir in env_dirs:
-            if os.path.isdir(env_dir):
-                for d in os.listdir(env_dir):
-                    d = os.path.join(env_dir, d)
-                    if os.path.isdir(d):
-                        environments.add(d)
-
-        return environments
-
-    if os.path.isdir(conda_base) and os.path.isfile(conda_exe):
-        print("Using miniconda installation from", conda_base)
-    else:
-        print("Installing miniconda into", conda_base)
-        install_miniconda(conda_base)
-
-    # verify consistency and check conda version
-    if not os.path.isfile(conda_exe):
-        sys.exit("Conda executable not found at " + conda_exe)
-
-    environments = get_environments()
-
-    conda_info = subprocess.check_output([conda_exe, "info", "--json"], env=clean_env)
-    if sys.version_info.major > 2:
-        conda_info = conda_info.decode("latin-1")
-    conda_info = json.loads(conda_info)
-    for env in environments:
-        if env not in conda_info["envs"]:
-            print("Consistency check:", env, "not in environments:")
-            print(conda_info["envs"])
-            warnings.warn(
-                """
-There is a mismatch between the conda settings in your home directory
-and what "conda info" is reporting. This is not a fatal error, but if
-an error is encountered, please check that your conda installation and
-environments exist and are working.
-""",
-                RuntimeWarning,
-            )
-
-    # identify packages required for environment
-    if os.name == "nt":
-        conda_platform = "windows"
-    elif sys.platform == "darwin":
-        conda_platform = "macos"
-    else:
-        conda_platform = "linux"
-    filename = os.path.join(
-        "modules",
-        "dials",
-        ".conda-envs",
-        "{platform}.txt".format(platform=conda_platform),
-    )
-    if not os.path.isfile(filename):
-        raise RuntimeError(
-            "The file {filename} is not available".format(filename=filename)
-        )
-
-    python_requirement = "conda-forge::python=%s.*" % python
-
-    # make a new environment directory
-    prefix = os.path.realpath("conda_base")
-
-    # install a new environment or update and existing one
-    if prefix in environments:
-        command = "install"
-        text_messages = ["Updating", "update of"]
-    else:
-        command = "create"
-        text_messages = ["Installing", "installation into"]
-    command_list = [
-        conda_exe,
-        command,
-        "--prefix",
-        prefix,
-        "--file",
-        filename,
-        "--yes",
-        "--channel",
-        "conda-forge",
-        "--override-channels",
-        python_requirement,
-    ]
-    if include_cctbx or cmake:
-        command_list.append("cctbx-nightly::cctbx-base=" + _prebuilt_cctbx_base)
-    if cmake:
-        command_list.extend(["pycbf", "cmake", "pre-commit"])
-    if os.name == "nt":
+    _run_conda_retry(command_list)
+    # If we wanted extra dependencies, and couldn't install them directly, run again
+    if extra_deps:
         command_list = [
-            "cmd.exe",
-            "/C",
-            " ".join(
-                [os.path.join(conda_base, "Scripts", "activate"), "base", "&&"]
-                + command_list
-            )
-            .replace("<", "^<")
-            .replace(">", "^>"),
-        ]
-    print(
-        "{text} dials environment from {filename} with Python {python}".format(
-            text=text_messages[0], filename=filename, python=python
-        )
-    )
-    for retry in range(5):
-        retry += 1
-        try:
-            run_command(
-                command=command_list,
-                workdir=".",
-            )
-        except Exception:
-            print(
-                """
-*******************************************************************************
-There was a failure in constructing the conda environment.
-Attempt {retry} of 5 will start {retry} minute(s) from {t}.
-*******************************************************************************
-""".format(
-                    retry=retry, t=time.asctime()
-                )
-            )
-            time.sleep(retry * 60)
-        else:
-            break
-    else:
-        sys.exit(
-            """
-The conda environment could not be constructed. Please check that there is a
-working network connection for downloading conda packages.
-"""
-        )
+            mamba,
+            "--no-env",
+            "--no-rc",
+            "--prefix",
+            prefix,
+            "--root-prefix",
+            mamba_prefix,
+            "install",
+            "--yes",
+            "--channel",
+            "conda-forge",
+            "--override-channels",
+            "mamba",
+        ] + extra_deps
+        _run_conda_retry(command_list)
+
     print("Completed {text}:\n  {prefix}".format(text=text_messages[1], prefix=prefix))
     with open(os.path.join(prefix, ".condarc"), "w") as fh:
         fh.write(
@@ -1013,7 +841,7 @@ def update_sources(options):
         repositories = {
             source.split("/")[1]: {
                 "base-repository": source,
-                "branch-local": "dials-3.17",
+                "branch-local": "dials-3.22",
             }
             for source, branch in (
                 ("dials/annlib_adaptbx", "master"),
@@ -1028,21 +856,16 @@ def update_sources(options):
                 ("xia2/xia2", "main"),
             )
         }
-        if options.prebuilt_cctbx:
-            repositories["cctbx_project"]["branch-local"] = (
-                "releases/" + _prebuilt_cctbx_base
-            )
-        else:
-            repositories["cctbx_project"] = {
-                "base-repository": "dials/cctbx_project",
-                "branch-local": "dials-3.17",
-            }
+        repositories["cctbx_project"] = {
+            "base-repository": "dials/cctbx_project",
+            "branch-local": "dials-3.22",
+        }
     else:
         # Only what we need for CMake
         repositories = {
             source.split("/")[1]: {
                 "base-repository": source,
-                "branch-local": "dials-3.17",
+                "branch-local": "dials-3.22",
             }
             for source, branch in (
                 ("dials/dxtbx", "main"),
@@ -1139,10 +962,10 @@ def run_tests():
     )
 
 
-def refresh_build(prebuilt_cctbx):
+def refresh_build():
     print("Running libtbx.refresh")
     run_indirect_command(
-        "libtbx.refresh" if prebuilt_cctbx else os.path.join("bin", "libtbx.refresh"),
+        os.path.join("bin", "libtbx.refresh"),
         args=[],
     )
 
@@ -1198,9 +1021,20 @@ def _get_cmake_exe():
 
 def refresh_build_cmake():
     conda_python = _get_base_python()
-    run_indirect_command(conda_python, ["-mpip", "install", "-e", "../modules/dxtbx"])
-    run_indirect_command(conda_python, ["-mpip", "install", "-e", "../modules/dials"])
-    run_indirect_command(conda_python, ["-mpip", "install", "-e", "../modules/xia2"])
+    run_indirect_command(
+        conda_python,
+        [
+            "-mpip",
+            "install",
+            "--no-deps",
+            "-e",
+            "../modules/dxtbx",
+            "-e",
+            "../modules/dials",
+            "-e",
+            "../modules/xia2",
+        ],
+    )
 
 
 def configure_build_cmake():
@@ -1255,7 +1089,6 @@ source {dist_root}/conda_base/etc/profile.d/conda.sh
 conda activate {dist_root}/conda_base
 """.format(
                     dist_root=os.getcwd(),
-                    build_lib=os.path.join(os.getcwd(), "build", "lib"),
                 )
             )
 
@@ -1283,13 +1116,14 @@ add_subdirectory(dials)
         [
             "../modules",
             "-DCMAKE_INSTALL_PREFIX=" + conda_base_root,
-            "-DHDF5_ROOT=" + conda_base_root,
+            "-DHDF5_DIR=" + conda_base_root,
+            "-DPython_ROOT_DIR=" + conda_base_root,
         ]
         + extra_args,
     )
 
 
-def configure_build(config_flags, prebuilt_cctbx):
+def configure_build(config_flags):
     conda_python = _get_base_python()
 
     # write a new-style environment setup script
@@ -1307,12 +1141,6 @@ def configure_build(config_flags, prebuilt_cctbx):
         config_flags.append("--cxxstd=c++14")
 
     print("Setting up build directory")
-    if prebuilt_cctbx:
-        run_indirect_command(
-            command="libtbx.configure",
-            args=["cbflib", "dials", "dxtbx", "prime", "xia2"],
-        )
-        return
 
     configcmd = [
         os.path.join("..", "modules", "cctbx_project", "libtbx", "configure.py"),
@@ -1331,6 +1159,7 @@ def configure_build(config_flags, prebuilt_cctbx):
         "xia2",
         "prime",
         "--skip_phenix_dispatchers",
+        "--use_environment",
     ] + config_flags
 
     run_indirect_command(
@@ -1339,20 +1168,16 @@ def configure_build(config_flags, prebuilt_cctbx):
     )
 
 
-def make_build(prebuilt_cctbx):
+def make_build():
     try:
         nproc = len(os.sched_getaffinity(0))
     except AttributeError:
         nproc = multiprocessing.cpu_count()
-    if prebuilt_cctbx:
-        command = "libtbx.scons"
-    else:
-        command = os.path.join("bin", "libtbx.scons")
+    command = os.path.join("bin", "libtbx.scons")
 
     run_indirect_command(command, args=["-j", str(nproc)])
-    if not prebuilt_cctbx:
-        # run build again to make sure everything is built
-        run_indirect_command(command, args=["-j", str(nproc)])
+    # run build again to make sure everything is built
+    run_indirect_command(command, args=["-j", str(nproc)])
 
 
 def make_build_cmake():
@@ -1360,7 +1185,15 @@ def make_build_cmake():
     if os.name == "nt":
         run_indirect_command(cmake_exe, ["--build", ".", "--config", "RelWithDebInfo"])
     else:
-        run_indirect_command(cmake_exe, ["--build", "."])
+        parallel = []
+        if "CMAKE_GENERATOR" not in os.environ:
+            if hasattr(os, "sched_getaffinity"):
+                cpu = os.sched_getaffinity(0)
+            else:
+                cpu = multiprocessing.cpu_count()
+            if isinstance(cpu, int):
+                parallel = ["--parallel", str(cpu)]
+        run_indirect_command(cmake_exe, ["--build", "."] + parallel)
 
 
 def repository_at_tag(string):
@@ -1435,8 +1268,8 @@ be passed separately with quotes to avoid confusion (e.g
     parser.add_argument(
         "--python",
         help="Install this minor version of Python (default: %(default)s)",
-        default="3.10",
-        choices=("3.9", "3.10", "3.11"),
+        default="3.11",
+        choices=("3.9", "3.10", "3.11", "3.12"),
     )
     parser.add_argument(
         "--branch",
@@ -1448,12 +1281,6 @@ be passed separately with quotes to avoid confusion (e.g
             "Specify as repository@branch, eg. 'dials@dials-next'"
         ),
     )
-    parser.add_argument(
-        "--conda",
-        help="Use miniconda instead of micromamba for the base installation step",
-        default=False,
-        action="store_true",
-    )
     parser.add_argument(
         "--clean",
         help="Remove temporary conda environments and package caches after installation",
@@ -1461,20 +1288,24 @@ be passed separately with quotes to avoid confusion (e.g
         action="store_true",
     )
     parser.add_argument(
-        # Use the conda-forge cctbx package instead of compiling cctbx from scratch
-        # This is not currently supported outside of CI builds
-        "--prebuilt-cctbx",
-        help=argparse.SUPPRESS,
-        default=False,
-        action="store_true",
+        "--libtbx",
+        help="Use the libtbx build system, compiling cctbx from scratch.",
+        action="store_false",
+        dest="cmake",
     )
     parser.add_argument(
         "--cmake",
-        help="Use the CMake build system. Implies --prebuilt-cctbx.",
         action="store_true",
+        dest="removed_cmake",
+        help=argparse.SUPPRESS,
     )
 
     options = parser.parse_args()
+    if options.removed_cmake or options.cmake:
+        # User passed the obsolete parameter
+        sys.exit(
+            "Error: --cmake is now the default, please pass --libtbx for release builds."
+        )
 
     print("Performing actions:", " ".join(options.actions))
 
@@ -1484,22 +1315,12 @@ be passed separately with quotes to avoid confusion (e.g
 
     # Build base packages
     if "base" in options.actions:
-        if options.conda:
-            install_conda(
-                options.python,
-                include_cctbx=options.prebuilt_cctbx,
-                cmake=options.cmake,
-            )
-            if options.clean:
-                shutil.rmtree(os.path.realpath("miniconda"))
-        else:
-            install_micromamba(
-                options.python,
-                include_cctbx=options.prebuilt_cctbx,
-                cmake=options.cmake,
-            )
-            if options.clean:
-                shutil.rmtree(os.path.realpath("micromamba"))
+        install_micromamba(
+            options.python,
+            cmake=options.cmake,
+        )
+        if options.clean:
+            shutil.rmtree(os.path.realpath("micromamba"))
 
     # Configure, make, get revision numbers
     if "build" in options.actions:
@@ -1508,9 +1329,9 @@ be passed separately with quotes to avoid confusion (e.g
             configure_build_cmake()
             make_build_cmake()
         else:
-            configure_build(options.config_flags, prebuilt_cctbx=options.prebuilt_cctbx)
-            make_build(prebuilt_cctbx=options.prebuilt_cctbx)
-            refresh_build(prebuilt_cctbx=options.prebuilt_cctbx)
+            configure_build(options.config_flags)
+            make_build()
+            refresh_build()
         install_precommit(options.cmake)
 
     # Tests, tests
