@@ -15,9 +15,11 @@ from dials.algorithms.clustering import plots as cluster_plotter
 from dials.algorithms.clustering.observers import uc_params_from_experiments
 from dials.algorithms.scaling.observers import make_merging_stats_plots
 from dials.array_family import flex
-from dials.command_line.stereographic_projection import calculate_projections
+from dials.command_line.stereographic_projection import (
+    calculate_projections,
+    projections_as_dict,
+)
 from dials.command_line.stereographic_projection import phil_scope as stereo_phil_scope
-from dials.command_line.stereographic_projection import projections_as_dict
 from dials.report.analysis import (
     format_statistics,
     make_merging_statistics_summary,
@@ -31,7 +33,6 @@ logger = logging.getLogger("dials")
 
 
 class MergeJSONCollector(object):
-
     initiated = False
     data = {}
 
@@ -52,9 +53,9 @@ class MergeJSONCollector(object):
 class MergingStatisticsData:
     experiments: ExperimentList
     scaled_miller_array: miller.array
-    reflections: Optional[
-        List[flex.reflection_table]
-    ] = None  # only needed if using this class like a script when making batch plots
+    reflections: Optional[List[flex.reflection_table]] = (
+        None  # only needed if using this class like a script when making batch plots
+    )
     merging_statistics_result: Optional[Type[dataset_statistics]] = None
     anom_merging_statistics_result: Optional[Type[dataset_statistics]] = None
     cut_merging_statistics_result: Optional[Type[dataset_statistics]] = None
@@ -71,9 +72,9 @@ class MergingStatisticsData:
             stats_summary += (
                 "\n"
                 "Resolution limit suggested from CC"
-                + "\u00BD"
+                + "\u00bd"
                 + " fit (limit CC"
-                + "\u00BD"
+                + "\u00bd"
                 + f"=0.3): {d_min:.2f}\n"
             )
         stats_summary += table_1_summary(
@@ -101,7 +102,6 @@ class MergingStatisticsData:
 
 
 def make_stereo_plots(experiments):
-
     orientation_graphs = OrderedDict()
     # now make stereo projections
     params = stereo_phil_scope.extract()
@@ -151,14 +151,14 @@ def generate_json_data(data: dict[float, MergingStatisticsData]) -> dict:
                 make_dano_plots({wl: stats.anomalous_amplitudes})["dF"]
             )
         if stats.merging_statistics_result:
-            json_data[wl_key][
-                "merging_stats"
-            ] = stats.merging_statistics_result.as_dict()
+            json_data[wl_key]["merging_stats"] = (
+                stats.merging_statistics_result.as_dict()
+            )
             json_data[wl_key]["table_1_stats"] = stats.table_1_stats()
             if stats.anom_merging_statistics_result:
-                json_data[wl_key][
-                    "merging_stats_anom"
-                ] = stats.anom_merging_statistics_result.as_dict()
+                json_data[wl_key]["merging_stats_anom"] = (
+                    stats.anom_merging_statistics_result.as_dict()
+                )
     if len(json_data) > 1:
         # create an overall summary table
         headers = [""] + ["Wavelength " + f"{wl:.5f}" + " Å" for wl in data.keys()]
@@ -273,7 +273,7 @@ https://strucbio.biologie.uni-konstanz.de/ccp4wiki/index.php?title=SHELX_C/D/E
                 "x": d_star_sq_bins,
                 "y": list(dFsdF),
                 "type": "scatter",
-                "name": "\u03BB" + f"={wave:.4f}",
+                "name": "\u03bb" + f"={wave:.4f}",
             }
         )
     if not data["dF"]["dano"]["data"]:
