@@ -21,6 +21,7 @@ from dials.algorithms.profile_model.ellipsoid.parameterisation import (
 )
 from dials.array_family import flex
 from dials.util import tabulate
+from dials_algorithms_profile_model_ellipsoid_ext import reflection_statistics, rse
 
 logger = logging.getLogger("dials")
 
@@ -174,8 +175,7 @@ def rotate_mat3_double(R, A):
 
 
 class ReflectionLikelihood(object):
-    def __init__(self, model, s0, sp, h, ctot, mobs, sobs):
-
+    def __init__(self, model, s0, sp, h, ctot, mobs, sobs, panel_id=0):
         # Save stuff
         modelstate = ReflectionModelState(model, s0, h)
         self.modelstate = modelstate
@@ -186,9 +186,11 @@ class ReflectionLikelihood(object):
         self.ctot = ctot
         self.mobs = mobs.reshape(2, 1)
         self.sobs = sobs
+        self.panel_id = panel_id
 
         # Compute the change of basis
         self.R = compute_change_of_basis_operation(self.s0, self.sp)  # const
+        self.R_cctbx = matrix.sqr(flex.double(self.R.tolist()))
         s2 = self.s0 + self.modelstate.get_r()
         # Rotate the mean vector
         self.mu = np.matmul(self.R, s2)
@@ -207,7 +209,6 @@ class ReflectionLikelihood(object):
         )
 
     def update(self):
-
         # The s2 vector
         s2 = self.s0 + self.modelstate.get_r()
         # Rotate the mean vector
@@ -389,8 +390,9 @@ class ReflectionLikelihood(object):
 
 
 class MaximumLikelihoodTarget(object):
-    def __init__(self, model, s0, sp_list, h_list, ctot_list, mobs_list, sobs_list):
-
+    def __init__(
+        self, model, s0, sp_list, h_list, ctot_list, mobs_list, sobs_list, panel_ids
+    ):
         # Check input
         assert len(h_list) == sp_list.shape[-1]
         assert len(h_list) == ctot_list.shape[-1]
@@ -412,6 +414,7 @@ class MaximumLikelihoodTarget(object):
                     ctot_list[i],
                     mobs_list[:, i],
                     sobs_list[:, :, i],
+                    panel_ids[i],
                 )
             )
 
@@ -438,34 +441,19 @@ class MaximumLikelihoodTarget(object):
         The RMSD in pixels
 
         """
-        mse = np.array([0.0, 0.0], dtype=np.float64)
+        mse_x = 0.0
+        mse_y = 0.0
         for i in range(len(self.data)):
-            R = self.data[i].R
-            mbar = self.data[i].conditional.mean()
-            xobs = self.data[i].mobs
+            R = self.data[i].R_cctbx
+            mbar = tuple(self.data[i].conditional.mean().flatten())
+            xobs = tuple(self.data[i].mobs.flatten())
             norm_s0 = self.data[i].norm_s0
-
-            s1 = np.matmul(
-                R.T,
-                np.array([mbar[0, 0], mbar[1, 0], norm_s0], dtype=np.float64).reshape(
-                    3, 1
-                ),
-            )
-            s3 = np.matmul(
-                R.T,
-                np.array([xobs[0, 0], xobs[1, 0], norm_s0], dtype=np.float64).reshape(
-                    3, 1
-                ),
-            )
-            s1 = matrix.col(flumpy.from_numpy(s1[:, 0]))
-            s3 = matrix.col(flumpy.from_numpy(s3[:, 0]))
-            xyzcal = self.model.experiment.detector[0].get_ray_intersection_px(s1)
-            xyzobs = self.model.experiment.detector[0].get_ray_intersection_px(s3)
-            r_x = xyzcal[0] - xyzobs[0]
-            r_y = xyzcal[1] - xyzobs[1]
-            mse += np.array([r_x**2, r_y**2])
-        mse /= len(self.data)
-        return np.sqrt(mse)
+            rse_i = rse(R, mbar, xobs, norm_s0, self.model.experiment.detector)
+            mse_x += rse_i[0]
+            mse_y += rse_i[1]
+        mse_x /= len(self.data)
+        mse_y /= len(self.data)
+        return np.sqrt(np.array([mse_x, mse_y]))
 
     def log_likelihood(self):
         """
@@ -557,7 +545,7 @@ class FisherScoringMaximumLikelihoodBase(object):
 
     """
 
-    def __init__(self, x0, max_iter=1000, tolerance=1e-7):
+    def __init__(self, x0, max_iter=1000, tolerance=1e-7, LL_tolerance=1e-6):
         """
         Configure the algorithm
 
@@ -569,6 +557,7 @@ class FisherScoringMaximumLikelihoodBase(object):
         self.x0 = matrix.col(x0)
         self.max_iter = max_iter
         self.tolerance = tolerance
+        self.LL_tolerance = LL_tolerance
 
     def solve(self):
         """
@@ -601,6 +590,8 @@ class FisherScoringMaximumLikelihoodBase(object):
             # Break the loop if the parameters change less than the tolerance
             if (x - x0).length() < self.tolerance:
                 break
+            if self.test_LL_convergence():
+                break
 
             # Update the parameter
             x0 = x
@@ -609,6 +600,16 @@ class FisherScoringMaximumLikelihoodBase(object):
         self.num_iter = it + 1
         self.parameters = x
 
+    def test_LL_convergence(self):
+        try:
+            l1 = self.history[-1]["likelihood"]
+            l2 = self.history[-2]["likelihood"]
+        except IndexError:
+            return False
+
+        test = abs(l1 - l2) < self.LL_tolerance
+        return test
+
     def solve_update_equation(self, S, I):
         """
         Solve the update equation using cholesky decomposition
@@ -675,8 +676,10 @@ class FisherScoringMaximumLikelihood(FisherScoringMaximumLikelihoodBase):
         ctot_list,
         mobs_list,
         sobs_list,
+        panel_ids,
         max_iter=1000,
         tolerance=1e-7,
+        LL_tolerance=1e-6,
     ):
         """
         Initialise the algorithm:
@@ -684,7 +687,10 @@ class FisherScoringMaximumLikelihood(FisherScoringMaximumLikelihoodBase):
         """
         # Initialise the super class
         super(FisherScoringMaximumLikelihood, self).__init__(
-            model.active_parameters, max_iter=max_iter, tolerance=tolerance
+            model.active_parameters,
+            max_iter=max_iter,
+            tolerance=tolerance,
+            LL_tolerance=LL_tolerance,
         )
 
         # Save the parameterisation
@@ -697,6 +703,7 @@ class FisherScoringMaximumLikelihood(FisherScoringMaximumLikelihoodBase):
         self.ctot_list = ctot_list
         self.mobs_list = mobs_list
         self.sobs_list = sobs_list
+        self.panel_ids = panel_ids
 
         # Store the parameter history
         self.history = []
@@ -709,6 +716,7 @@ class FisherScoringMaximumLikelihood(FisherScoringMaximumLikelihoodBase):
             self.ctot_list,
             self.mobs_list,
             self.sobs_list,
+            self.panel_ids,
         )
 
         # Print initial
@@ -810,7 +818,9 @@ class FisherScoringMaximumLikelihood(FisherScoringMaximumLikelihoodBase):
 
         # Get some matrices
         U = self.model.U_matrix.flatten()
-        M = self.model.mosaicity_covariance_matrix.flatten()
+        M = (
+            self.model._M_parameterisation.sigma().flatten()
+        )  # mosaicity_covariance_matrix.flatten()
 
         # Print some information
         format_string1 = "  Unit cell: (%.3f, %.3f, %.3f, %.3f, %.3f, %.3f)"
@@ -844,6 +854,19 @@ class FisherScoringMaximumLikelihood(FisherScoringMaximumLikelihoodBase):
                     ]
                 )
             )
+            if self.model.is_mosaic_spread_angular:
+                MA = self.model._M_parameterisation.sigma_A().flatten()
+                logger.info(
+                    "\n".join(
+                        [
+                            "",
+                            "  Sigma M angular",
+                            format_string3 % tuple(MA[0:3]),
+                            format_string3 % tuple(MA[3:6]),
+                            format_string3 % tuple(MA[6:9]),
+                        ]
+                    )
+                )
 
         logger.info(
             "\n".join(
@@ -877,7 +900,7 @@ class Refiner(object):
 
     """
 
-    def __init__(self, state, data):
+    def __init__(self, state, data, max_iter=1000, LL_tolerance=1e-6):
         """
         Set the data and initial parameters
 
@@ -888,8 +911,11 @@ class Refiner(object):
         self.ctot_list = data.ctot_list
         self.mobs_list = data.mobs_list
         self.sobs_list = data.sobs_list
+        self.panel_ids = data.panel_ids
         self.state = state
         self.history = []
+        self.max_iter = max_iter
+        self.LL_tolerance = LL_tolerance
 
     def refine(self):
         """
@@ -922,6 +948,9 @@ class Refiner(object):
             self.ctot_list,
             self.mobs_list,
             self.sobs_list,
+            self.panel_ids,
+            max_iter=self.max_iter,
+            LL_tolerance=self.LL_tolerance,
         )
 
         # Solve the maximum likelihood equations
@@ -947,9 +976,19 @@ class Refiner(object):
         # Print the eigen values and vectors of sigma_m
         if not self.state.is_mosaic_spread_fixed:
             logger.info("\nDecomposition of Sigma_M:")
-            print_eigen_values_and_vectors(
-                matrix.sqr(flumpy.from_numpy(self.state.mosaicity_covariance_matrix))
+            print_eigen_values_and_vectors_static(
+                matrix.sqr(
+                    flumpy.from_numpy(self.state._M_parameterisation.sigma().flatten())
+                )
             )
+            if self.state.is_mosaic_spread_angular:
+                print_eigen_values_and_vectors_angular(
+                    matrix.sqr(
+                        flumpy.from_numpy(
+                            self.state._M_parameterisation.sigma_A()[:2, :2].flatten()
+                        )
+                    )
+                )
 
         # Save the history
         self.history = self.ml.history
@@ -978,7 +1017,7 @@ class RefinerData(object):
 
     """
 
-    def __init__(self, s0, sp_list, h_list, ctot_list, mobs_list, sobs_list):
+    def __init__(self, s0, sp_list, h_list, ctot_list, mobs_list, sobs_list, panel_ids):
         """
         Init the data
 
@@ -991,6 +1030,7 @@ class RefinerData(object):
         self.ctot_list = ctot_list
         self.mobs_list = mobs_list
         self.sobs_list = sobs_list
+        self.panel_ids = panel_ids
 
     @classmethod
     def from_reflections(self, experiment, reflections):
@@ -1015,48 +1055,15 @@ class RefinerData(object):
             "Computing observed covariance for %d reflections" % len(reflections)
         )
         s0_length = norm(s0)
-        assert len(experiment.detector) == 1
-        panel = experiment.detector[0]
+
         sbox = reflections["shoebox"]
-        xyzobs = reflections["xyzobs.px.value"]
-        for r in range(len(reflections)):
-
-            # The vector to the pixel centroid
-            sp = np.array(
-                panel.get_pixel_lab_coord(xyzobs[r][0:2]), dtype=np.float64
-            ).reshape(3, 1)
-            sp *= s0_length / norm(sp)
-
-            # Compute change of basis
-            R = compute_change_of_basis_operation(s0, sp)
-
-            # Get data and compute total counts
-            data = sbox[r].data
-            mask = sbox[r].mask
-            bgrd = sbox[r].background
-
-            # Get array of vectors
-            i0 = sbox[r].bbox[0]
-            j0 = sbox[r].bbox[2]
-            assert data.all()[0] == 1
-            X = np.zeros(shape=(data.all()[1:]) + (2,), dtype=np.float64)
-            C = np.zeros(shape=data.all()[1:], dtype=np.float64)
-            ctot = 0
-            for j in range(data.all()[1]):
-                for i in range(data.all()[2]):
-                    c = data[0, j, i] - bgrd[0, j, i]
-                    if mask[0, j, i] & (1 | 4) == (1 | 4) and c > 0:
-                        ctot += c
-                        ii = i + i0
-                        jj = j + j0
-                        s = np.array(
-                            panel.get_pixel_lab_coord((ii + 0.5, jj + 0.5)),
-                            dtype=np.float64,
-                        ).reshape(3, 1)
-                        s *= s0_length / norm(s)
-                        e = np.matmul(R, s)
-                        X[j, i, :] = e[0:2, 0]
-                        C[j, i] = c
+        for r, (panel_id, xyz) in enumerate(
+            zip(reflections["panel"], reflections["xyzobs.px.value"])
+        ):
+            panel = experiment.detector[panel_id]
+            sp, ctot, xbar, Sobs = reflection_statistics(
+                panel, xyz, s0_length, experiment.beam.get_s0(), sbox[r]
+            )
 
             # Check we have a sensible number of counts
             if ctot <= 0:
@@ -1064,32 +1071,16 @@ class RefinerData(object):
                     "Strong spot found with <= 0 counts! Check spotfinding results"
                 )
 
-            # Compute the mean vector
-            C = np.expand_dims(C, axis=2)
-            xbar = C * X
-            xbar = xbar.reshape(-1, xbar.shape[-1]).sum(axis=0)
-            xbar /= ctot
-
-            xbar = xbar.reshape(2, 1)
-
-            # Compute the covariance matrix
-            Sobs = np.array([[0.0, 0.0], [0.0, 0.0]], dtype=np.float64)
-            for j in range(X.shape[0]):
-                for i in range(X.shape[1]):
-                    x = np.array(X[j, i], dtype=np.float64).reshape(2, 1)
-                    Sobs += np.matmul(x - xbar, (x - xbar).T) * C[j, i, 0]
-
-            Sobs /= ctot
-            if (Sobs[0, 0] <= 0) or (Sobs[1, 1] <= 0):
+            if (Sobs[0] <= 0) or (Sobs[3] <= 0):
                 raise BadSpotForIntegrationException(
                     "Strong spot variance <= 0. Check spotfinding results"
                 )
 
             # Add to the lists
-            sp_list[:, r] = sp[:, 0]
+            sp_list[:, r] = sp  # [:, 0]
             ctot_list[r] = ctot
-            mobs_list[:, r] = xbar[:, 0]
-            Sobs_list[:, :, r] = Sobs
+            mobs_list[:, r] = xbar  # [:, 0]
+            Sobs_list[:, :, r] = np.array([Sobs[0:2], Sobs[2:]], dtype=np.float64)
 
         # Print some information
         logger.info("")
@@ -1142,7 +1133,9 @@ class RefinerData(object):
         )
 
         # Return the profile refiner data
-        return RefinerData(s0, sp_list, h_list, ctot_list, mobs_list, Sobs_list)
+        return RefinerData(
+            s0, sp_list, h_list, ctot_list, mobs_list, Sobs_list, reflections["panel"]
+        )
 
 
 def print_eigen_values_and_vectors_of_observed_covariance(A, s0):
@@ -1167,7 +1160,34 @@ def print_eigen_values_and_vectors_of_observed_covariance(A, s0):
     logger.info("C2: %.5f degrees" % (sqrt(L[3]) * (180.0 / pi) / s0.length()))
 
 
-def print_eigen_values_and_vectors(A):
+def print_eigen_values_and_vectors_static(A):
+    """
+    Print the eigen values and vectors of a matrix
+
+    """
+
+    # Compute the eigen decomposition of the covariance matrix
+    eigen_decomposition = linalg.eigensystem.real_symmetric(A.as_flex_double_matrix())
+    eigen_values = eigen_decomposition.values()
+
+    # Print the matrix eigen values
+    logger.info(
+        f"\n Eigen Values:\n{print_matrix(matrix.diag(eigen_values), indent=2)}\n"
+    )
+    logger.info(
+        f"\n Eigen Vectors:\n{print_matrix(matrix.sqr(eigen_decomposition.vectors()), indent=2)}\n"
+    )
+    logger.info(
+        f"""
+ Invariant crystal mosaicity:
+ M1 : {eigen_values[0]**0.5:.5f} Å⁻¹
+ M2 : {eigen_values[1]**0.5:.5f} Å⁻¹
+ M3 : {eigen_values[2]**0.5:.5f} Å⁻¹
+"""
+    )
+
+
+def print_eigen_values_and_vectors_angular(A):
     """
     Print the eigen values and vectors of a matrix
 
@@ -1187,12 +1207,9 @@ def print_eigen_values_and_vectors(A):
 
     mosaicity = mosaicity_from_eigen_decomposition(eigen_values)
     logger.info(
-        f"""
- Mosaicity in degrees equivalent units:
- M1 : {mosaicity[0]:.5f} degrees
- M2 : {mosaicity[1]:.5f} degrees
- M3 : {mosaicity[2]:.5f} degrees
-"""
+        """
+ Angular mosaicity in degrees equivalent units:\n"""
+        + "\n".join(f" M{i+1} : {m:.5f} degrees" for i, m in enumerate(mosaicity))
     )
 
 
