@@ -9,7 +9,7 @@ from libtbx.phil import parse
 
 from dials.array_family import flex
 from dials.util import tabulate
-from dials.util.mp import available_cores
+from dials.util.system import CPU_COUNT
 
 logger = logging.getLogger(__name__)
 
@@ -27,7 +27,6 @@ class CentroidOutlier:
         block_width=None,
         nproc=1,
     ):
-
         # column names of the data in which to look for outliers
         if cols is None:
             cols = ["x_resid", "y_resid", "phi_resid"]
@@ -248,7 +247,6 @@ class CentroidOutlier:
 
         # loop over the completed jobs
         for i, job in enumerate(jobs3):
-
             iexp = job["id"]
             ipanel = job["panel"]
             nref = len(job["indices"])
@@ -299,7 +297,6 @@ class CentroidOutlier:
 
         msg = None
         if nref >= self._min_num_obs:
-
             # get the subset of data as a list of columns
             cols = [data[col] for col in self._cols]
 
@@ -477,7 +474,6 @@ phil_scope = parse(phil_str)
 class CentroidOutlierFactory:
     @staticmethod
     def from_parameters_and_colnames(params, colnames):
-
         # id the relevant scope for the requested method
         method = params.outlier.algorithm
         if method == "null":
@@ -522,7 +518,7 @@ class CentroidOutlierFactory:
             params.outlier.block_width = None
 
         if params.outlier.nproc is libtbx.Auto:
-            params.outlier.nproc = available_cores()
+            params.outlier.nproc = CPU_COUNT
             logger.info("Setting outlier.nproc={}".format(params.outlier.nproc))
 
         od = outlier_detector(
@@ -539,7 +535,6 @@ class CentroidOutlierFactory:
 
 
 if __name__ == "__main__":
-
     # test construction
     params = phil_scope.extract()
     params.outlier.algorithm = "tukey"
