@@ -4,6 +4,7 @@ import logging
 import re
 
 import libtbx
+from dxtbx.model import PolychromaticBeam
 from libtbx.phil import parse
 
 from dials.algorithms.refinement import DialsRefineConfigError
@@ -15,11 +16,12 @@ from dials.algorithms.refinement.parameterisation.scan_varying_model_parameters
     phil_str as sv_phil_str,
 )
 from dials.algorithms.refinement.refinement_helpers import string_sel
+from dials.algorithms.refinement.reflection_manager import LaueReflectionManager
 from dials.algorithms.refinement.restraints.restraints_parameterisation import (
     uc_phil_str as uc_restraints_phil_str,
 )
 
-from .beam_parameters import BeamParameterisation
+from .beam_parameters import BeamParameterisation, LaueBeamParameterisation
 from .crystal_parameters import (
     CrystalOrientationParameterisation,
     CrystalUnitCellParameterisation,
@@ -31,6 +33,7 @@ from .detector_parameters import (
 )
 from .goniometer_parameters import GoniometerParameterisation
 from .prediction_parameters import (
+    LauePredictionParameterisation,
     XYPhiPredictionParameterisation,
     XYPhiPredictionParameterisationSparse,
 )
@@ -317,7 +320,6 @@ def _filter_parameter_names(parameterisation):
 
 # Helper function to perform centroid analysis
 def _centroid_analysis(options, experiments, reflection_manager):
-
     analysis = None
     if not options.scan_varying:
         return analysis
@@ -433,8 +435,15 @@ def _parameterise_beams(options, experiments, analysis):
                 experiment_ids=exp_ids,
             )
         else:
-            # Parameterise scan static beam, passing the goniometer
-            beam_param = BeamParameterisation(beam, goniometer, experiment_ids=exp_ids)
+            if isinstance(beam, PolychromaticBeam):
+                beam_param = LaueBeamParameterisation(
+                    beam, goniometer, experiment_ids=exp_ids
+                )
+            else:
+                # Parameterise scan static beam, passing the goniometer
+                beam_param = BeamParameterisation(
+                    beam, goniometer, experiment_ids=exp_ids
+                )
 
         # Set the model identifier to name the parameterisation
         beam_param.model_identifier = f"Beam{ibeam + 1}"
@@ -454,7 +463,9 @@ def _parameterise_beams(options, experiments, analysis):
                 fix_list.append("Mu1")
             if "out_spindle_plane" in options.beam.fix:
                 fix_list.append("Mu2")
-            if "wavelength" in options.beam.fix:
+            if "wavelength" in options.beam.fix and not isinstance(
+                beam, PolychromaticBeam
+            ):
                 fix_list.append("nu")
 
         if fix_list:
@@ -818,10 +829,16 @@ def build_prediction_parameterisation(
     analysis = _centroid_analysis(options, experiments, reflection_manager)
 
     # Parameterise each unique model
-    beam_params = _parameterise_beams(options, experiments, analysis)
     xl_ori_params, xl_uc_params = _parameterise_crystals(options, experiments, analysis)
     det_params = _parameterise_detectors(options, experiments, analysis)
     gon_params = _parameterise_goniometers(options, experiments, analysis)
+    beam_params = _parameterise_beams(options, experiments, analysis)
+
+    if isinstance(reflection_manager, LaueReflectionManager):
+        PredParam = LauePredictionParameterisation
+        return PredParam(
+            experiments, det_params, beam_params, xl_ori_params, xl_uc_params
+        )
 
     # Build the prediction equation parameterisation
     if do_stills:  # doing stills
