@@ -64,7 +64,6 @@ class PredictionParameterisation:
         xl_unit_cell_parameterisations=None,
         goniometer_parameterisations=None,
     ):
-
         if detector_parameterisations is None:
             detector_parameterisations = []
         if beam_parameterisations is None:
@@ -147,7 +146,6 @@ class PredictionParameterisation:
         return self._goniometer_parameterisations
 
     def _len(self):
-
         length = 0
         for model in self._detector_parameterisations:
             length += model.num_free()
@@ -328,7 +326,6 @@ class PredictionParameterisation:
 
         # Populate values in these arrays
         for iexp, exp in enumerate(self._experiments):
-
             sel = reflections["id"] == iexp
             isel = sel.iselection()
             self._experiment_to_idx.append(isel)
@@ -418,8 +415,13 @@ class PredictionParameterisation:
             sel = panels == ipanel
             D.set_selected(sel, D_mat)
 
+        if "s0" in reflections:
+            s0 = reflections["s0"]
+        else:
+            s0 = experiment.beam.get_s0()
+
         result = {
-            "s0": experiment.beam.get_s0(),
+            "s0": s0,
             "U": matrix.sqr(experiment.crystal.get_U()),
             "B": matrix.sqr(experiment.crystal.get_B()),
             "D": D,
@@ -445,7 +447,6 @@ class PredictionParameterisation:
         D = self._D.select(isel)
 
         if dd_ddet_p is None:
-
             # get the derivatives of detector d matrix for this panel
             dd_ddet_p = parameterisation.get_ds_dp(
                 multi_state_elt=panel_id, use_none_as_null=True
@@ -492,7 +493,6 @@ class PredictionParameterisation:
 
         # loop over the detector parameterisations
         for dp in self._detector_parameterisations:
-
             # Determine (sub)set of reflections affected by this parameterisation
             isel = flex.size_t()
             for exp_id in dp.get_experiment_ids():
@@ -511,7 +511,6 @@ class PredictionParameterisation:
 
             # loop through the panels in this detector
             for panel_id, _ in enumerate(detector):
-
                 # get the right subset of array indices to set for this panel
                 sub_isel = isel.select(panel == panel_id)
                 if len(sub_isel) == 0:
@@ -573,7 +572,6 @@ class PredictionParameterisation:
     ):
         # loop over the parameterisations
         for p in parameterisations:
-
             # Determine (sub)set of reflections affected by this parameterisation
             isel = flex.size_t()
             for exp_id in p.get_experiment_ids():
@@ -710,7 +708,6 @@ class SparseGradientVectorMixin:
 
 
 class XYPhiPredictionParameterisation(PredictionParameterisation):
-
     _grad_names = ("dX_dp", "dY_dp", "dphi_dp")
 
     def _local_setup(self, reflections):
@@ -786,7 +783,6 @@ class XYPhiPredictionParameterisation(PredictionParameterisation):
         D = self._D.select(isel)
 
         if ds0_dbeam_p is None:
-
             # get the derivatives of the beam vector wrt the parameters
             ds0_dbeam_p = parameterisation.get_ds_dp(use_none_as_null=True)
 
@@ -800,7 +796,6 @@ class XYPhiPredictionParameterisation(PredictionParameterisation):
 
         # loop through the parameters
         for der in ds0_dbeam_p:
-
             if der is None:
                 dphi_dp.append(None)
                 dpv_dp.append(None)
@@ -903,7 +898,6 @@ class XYPhiPredictionParameterisation(PredictionParameterisation):
         D = self._D.select(isel)
 
         if dS_dgon_p is None:
-
             # get derivatives of the setting matrix S wrt the parameters
             dS_dgon_p = [
                 None if der is None else flex.mat3_double(len(isel), der.elems)
@@ -915,7 +909,6 @@ class XYPhiPredictionParameterisation(PredictionParameterisation):
 
         # loop through the parameters
         for der in dS_dgon_p:
-
             if der is None:
                 dphi_dp.append(None)
                 dpv_dp.append(None)
@@ -967,3 +960,235 @@ class XYPhiPredictionParameterisationSparse(
     @staticmethod
     def _extend_gradient_vectors(results, m, n, keys=("dX_dp", "dY_dp", "dZ_dp")):
         return SparseGradientVectorMixin._extend_gradient_vectors(results, m, n, keys)
+
+
+class LauePredictionParameterisation(PredictionParameterisation):
+    """A basic extension to PredictionParameterisation for Laue data,
+    where gradients for the wavelength of each reflection are also considered."""
+
+    _grad_names = ("dX_dp", "dY_dp", "dwavelength_dp")
+
+    def __init__(self, *args, **kwargs):
+        super().__init__(*args, **kwargs)
+        return
+
+    def _local_setup(self, reflections):
+        self._wavelength = reflections["wavelength_cal"]
+        self._r = self._setting_rotation * self._fixed_rotation * self._UB * self._h
+        self._s0 = reflections["s0_cal"]
+        self._e_X_r = (self._setting_rotation * self._axis).cross(self._r)
+        self._e_r_s0 = (self._e_X_r).dot(self._s0)
+        self._ds0_dbeam_p = None
+        return
+
+    def get_gradients(self, reflections, callback=None):
+        """Calculate gradients of the prediction formula with respect to each
+        of the parameters of the contained models, for all of the reflections.
+        This method sets up required quantities relevant to the current step of
+        refinement and then loops over the parameterisations of each type extending
+        a results list each time."""
+
+        # Set up arrays of quantities of interest for each reflection
+        self._nref = len(reflections)
+        self._D = flex.mat3_double(self._nref)
+        self._s0 = flex.vec3_double(self._nref)
+        self._U = flex.mat3_double(self._nref)
+        self._B = flex.mat3_double(self._nref)
+        self._axis = flex.vec3_double(self._nref, (0.0, 1.0, 0.0))
+        self._fixed_rotation = flex.mat3_double(self._nref, (1, 0, 0, 0, 1, 0, 0, 0, 1))
+        self._setting_rotation = flex.mat3_double(
+            self._nref, (1, 0, 0, 0, 1, 0, 0, 0, 1)
+        )
+
+        # Set up experiment to index mapping
+        self._experiment_to_idx = []
+
+        # Populate values in these arrays
+        for iexp, exp in enumerate(self._experiments):
+            sel = reflections["id"] == iexp
+            isel = sel.iselection()
+            self._experiment_to_idx.append(isel)
+            subref = reflections.select(sel)
+            states = self._get_model_data_for_experiment(exp, subref)
+
+            self._D.set_selected(sel, states["D"])
+            self._s0.set_selected(sel, states["s0"])
+            self._U.set_selected(sel, states["U"])
+            self._B.set_selected(sel, states["B"])
+            if exp.goniometer:
+                self._setting_rotation.set_selected(sel, states["S"])
+                self._axis.set_selected(sel, exp.goniometer.get_rotation_axis_datum())
+                self._fixed_rotation.set_selected(
+                    sel, exp.goniometer.get_fixed_rotation()
+                )
+
+        # Other derived values
+        self._h = reflections["miller_index"].as_vec3_double()
+        self._UB = self._U * self._B
+        self._s1 = reflections["s1"]
+        self._pv = self._D * self._s1  # 'projection vector' for the ray along s1.
+
+        # Quantities derived from pv, precalculated for efficiency
+        u, v, w = self._pv.parts()
+        assert w.all_ne(0)
+        self._w_inv = 1.0 / w
+        self._u_w_inv = u * self._w_inv
+        self._v_w_inv = v * self._w_inv
+
+        # Reset a pointer to the parameter number
+        self._iparam = 0
+
+        # Do additional setup specified by derived classes
+        self._local_setup(reflections)
+
+        # Set up empty list in which to store gradients
+        results = []
+
+        # loop over detector parameterisations and extend results
+        results = self._grads_detector_loop(reflections, results, callback=callback)
+
+        # loop over the beam parameterisations and extend results
+        results = self._grads_beam_loop(reflections, results, callback=callback)
+
+        # loop over the crystal orientation parameterisations and extend results
+        results = self._grads_xl_orientation_loop(
+            reflections, results, callback=callback
+        )
+
+        # loop over the crystal unit cell parameterisations and extend results
+        results = self._grads_xl_unit_cell_loop(reflections, results, callback=callback)
+
+        # loop over the goniometer parameterisations and extend results
+        results = self._grads_goniometer_loop(reflections, results, callback=callback)
+
+        return results
+
+    def _beam_derivatives(
+        self, isel, parameterisation=None, ds0_dbeam_p=None, reflections=None
+    ):
+        """helper function to extend the derivatives lists by derivatives of the
+        beam parameterisations."""
+
+        # Get required data
+        r = self._r.select(isel)
+        e_X_r = self._e_X_r.select(isel)
+        e_r_s0 = self._e_r_s0.select(isel)
+        D = self._D.select(isel)
+
+        if ds0_dbeam_p is None:
+            # get the derivatives of the beam vector wrt the parameters
+            ds0_dbeam_p = parameterisation.get_ds_dp(use_none_as_null=True)
+
+            ds0_dbeam_p = [
+                None if e is None else flex.vec3_double(len(r), e.elems)
+                for e in ds0_dbeam_p
+            ]
+            self._ds0_dbeam_p = ds0_dbeam_p
+
+        dphi_dp = []
+        dpv_dp = []
+
+        # loop through the parameters
+        for der in ds0_dbeam_p:
+            if der is None:
+                dphi_dp.append(None)
+                dpv_dp.append(None)
+                continue
+
+            # calculate the derivative of phi for this parameter
+            dphi = (der.dot(r) / e_r_s0) * -1.0
+            dphi_dp.append(dphi)
+
+            # calculate the derivative of pv for this parameter
+            dpv_dp.append(D * (e_X_r * dphi + der))
+
+        return dpv_dp, dphi_dp
+
+    def _xl_derivatives(self, isel, derivatives, b_matrix, parameterisation=None):
+        """helper function to extend the derivatives lists by derivatives of
+        generic parameterisations."""
+
+        # Get required data
+        h = self._h.select(isel)
+        if b_matrix:
+            B = self._B.select(isel)
+        else:
+            U = self._U.select(isel)
+        D = self._D.select(isel)
+        wavelength = self._wavelength.select(isel)
+        fixed_rotation = self._fixed_rotation.select(isel)
+        setting_rotation = self._setting_rotation.select(isel)
+        r = self._r.select(isel)
+
+        if derivatives is None:
+            # get derivatives of the B/U matrix wrt the parameters
+            derivatives = [
+                None if der is None else flex.mat3_double(len(isel), der.elems)
+                for der in parameterisation.get_ds_dp(use_none_as_null=True)
+            ]
+
+        dpv_dp = []
+        dwavelength_dp = []
+
+        # loop through the parameters
+        for idx, der in enumerate(derivatives):
+            if der is None:
+                dpv_dp.append(None)
+                dwavelength_dp.append(None)
+                continue
+
+            # calculate the derivative of r for this parameter
+            if b_matrix:
+                dr = setting_rotation * fixed_rotation * der * B * h
+            else:
+                dr = setting_rotation * fixed_rotation * U * der * h
+
+            unit_s0 = flex.vec3_double(len(dr), self._experiments[0].beam.get_unit_s0())
+            r_dot_r = r.dot(r)
+            dwavelength = -2 * unit_s0.dot(
+                ((dr * r_dot_r) - ((2 * r) * dr.dot(r))) / (r_dot_r**2)
+            )
+            dwavelength_dp.append(dwavelength)
+
+            # calculate the derivative of pv for this parameter
+            dpv_dp.append(D * (dr - (unit_s0 / wavelength**2) * dwavelength))
+
+        return dpv_dp, dwavelength_dp
+
+    def _xl_orientation_derivatives(
+        self, isel, parameterisation=None, dU_dxlo_p=None, reflections=None
+    ):
+        """helper function to extend the derivatives lists by derivatives of the
+        crystal orientation parameterisations"""
+        return self._xl_derivatives(
+            isel, dU_dxlo_p, b_matrix=True, parameterisation=parameterisation
+        )
+
+    def _xl_unit_cell_derivatives(
+        self, isel, parameterisation=None, dB_dxluc_p=None, reflections=None
+    ):
+        """helper function to extend the derivatives lists by
+        derivatives of the crystal unit cell parameterisations"""
+        return self._xl_derivatives(
+            isel, dB_dxluc_p, b_matrix=False, parameterisation=parameterisation
+        )
+
+    @staticmethod
+    def _calc_dX_dp_and_dY_dp_from_dpv_dp(w_inv, u_w_inv, v_w_inv, dpv_dp):
+        """helper function to calculate positional derivatives from
+        dpv_dp using the quotient rule"""
+
+        dX_dp = []
+        dY_dp = []
+
+        for der in dpv_dp:
+            if der is None:
+                dX_dp.append(None)
+                dY_dp.append(None)
+            else:
+                du_dp, dv_dp, dw_dp = der.parts()
+
+                dX_dp.append(w_inv * (du_dp - dw_dp * u_w_inv))
+                dY_dp.append(w_inv * (dv_dp - dw_dp * v_w_inv))
+
+        return dX_dp, dY_dp
