@@ -37,9 +37,7 @@ class active_parameter_manager:
         n_cumul_params = 0
         for component, obj in components.items():
             if component in selection_list:
-                assert hasattr(
-                    obj, "parameters"
-                ), """component object must have the
+                assert hasattr(obj, "parameters"), """component object must have the
           attribute 'parameters' for access to the component parameters."""
                 self.x.extend(obj.free_parameters)
                 n_params = len(obj.free_parameters)
@@ -204,7 +202,6 @@ class multi_active_parameter_manager(TargetInterface):
 
 
 class shared_active_parameter_manager(multi_active_parameter_manager):
-
     """Class to enforce sharing of model components.
 
     Intercept calls to a multi_apm, to override set_params calls and manage
