@@ -6,9 +6,6 @@ This module provides a base class for symmetry determination algorithms.
 from __future__ import annotations
 
 import logging
-
-logger = logging.getLogger(__name__)
-
 from io import StringIO
 
 import libtbx
@@ -21,6 +18,16 @@ from scitbx.array_family import flex
 from dials.util import resolution_analysis
 from dials.util.normalisation import quasi_normalisation
 
+logger = logging.getLogger(__name__)
+
+
+def median_unit_cell(experiments):
+    uc_params = [flex.double() for i in range(6)]
+    for c in experiments.crystals():
+        for i, p in enumerate(c.get_unit_cell().parameters()):
+            uc_params[i].append(p)
+    return uctbx.unit_cell(parameters=[flex.median(p) for p in uc_params])
+
 
 class symmetry_base:
     """Base class for symmetry analysis."""
