@@ -11,7 +11,7 @@ from orderedset import OrderedSet
 
 import dxtbx.model.compare as compare
 import libtbx.phil
-from dxtbx.imageset import ImageGrid, ImageSequence
+from dxtbx.imageset import ImageSequence
 from dxtbx.model.experiment_list import (
     Experiment,
     ExperimentList,
@@ -149,13 +149,6 @@ phil_scope = libtbx.phil.parse(
       .type = bool
       .help = "If False, raise an error if multiple sequences are found"
 
-    as_grid_scan = False
-      .type = bool
-      .help = "Import as grid scan"
-
-    grid_size = None
-      .type = ints(size=2)
-      .help = "If importing as a grid scan set the size"
   }
 
   include scope dials.util.options.format_phil_scope
@@ -210,7 +203,6 @@ def _extract_or_read_imagesets(params):
 
     # Check we have some filenames
     if len(experiments) == 0:
-
         # FIXME Should probably make this smarter since it requires editing here
         # and in dials.import phil scope
         try:
@@ -224,7 +216,6 @@ def _extract_or_read_imagesets(params):
         # Check if a template has been set and print help if not, otherwise try to
         # import the images based on the template input
         if len(params.input.template) > 0:
-
             experiments = ExperimentListFactory.from_templates(
                 params.input.template,
                 image_range=params.geometry.scan.image_range,
@@ -496,16 +487,12 @@ class MetaDataUpdater:
         """
         # Import the lookup data
         lookup = self.import_lookup_data(self.params)
-        # Convert all to ImageGrid
-        if self.params.input.as_grid_scan:
-            imageset_list = self.convert_to_grid_scan(imageset_list, self.params)
 
         # Create the experiments
         experiments = ExperimentList()
 
         # Loop through imagesets
         for imageset in imageset_list:
-
             # Set the external lookups
             imageset = self.update_lookup(imageset, lookup)
 
@@ -540,7 +527,7 @@ class MetaDataUpdater:
             ].count(None) == 1:
                 raise Sorry(
                     """
-          Only 1 offset map is set. Need to set both dx and d
+          Only 1 offset map is set. Need to set both dx and dy
         """
                 )
 
@@ -771,19 +758,6 @@ class MetaDataUpdater:
             dy=Item(data=dy, filename=dy_filename),
         )
 
-    def convert_to_grid_scan(self, imageset_list, params):
-        """
-        Convert the imagesets to grid scans
-        """
-        if params.input.grid_size is None:
-            raise Sorry("The input.grid_size parameter is required")
-        result = []
-        for imageset in imageset_list:
-            result.append(
-                ImageGrid.from_imageset(imageset.as_imageset(), params.input.grid_size)
-            )
-        return result
-
 
 def print_sequence_diff(sequence1, sequence2, params):
     """
@@ -827,7 +801,6 @@ def assert_single_sequence(experiments, params):
     ]
 
     if len(sequences) > 1:
-
         # Print some info about multiple sequences
         diagnose_multiple_sequences(sequences, params)
 
@@ -964,7 +937,6 @@ def do_import(
 
     # Print out info for all experiments
     for experiment in experiments:
-
         # Print some experiment info - override the output of image range
         # if appropriate
         image_range = params.geometry.scan.image_range
