@@ -2,7 +2,6 @@ from __future__ import annotations
 
 import logging
 import sys
-from io import StringIO
 
 from iotbx.phil import parse
 from libtbx import Auto
@@ -46,6 +45,11 @@ input is an models.expt file.
 XDS format exports a models.expt file as XDS.INP and XPARM.XDS files. If a
 reflection file is given it will be exported as a SPOT.XDS file.
 
+SHELX format exports intensity data in HKLF 4 format for use in the SHELX suite
+of programs. As this file format does not contain unit cell parameters or
+symmetry information a minimal instruction file is also written. Optionally the
+expected contents of the asymmetric unit can be added to this instruciton file.
+
 PETS format exports intensity data and diffraction data in the CIF format
 used by PETS. This is primarily intended to produce files suitable for
 dynamic diffraction refinement using Jana2020, which requires this format.
@@ -73,6 +77,11 @@ Examples::
   dials.export indexed.refl format=xds
   dials.export models.expt format=xds
   dials.export models.expt indexed.refl format=xds
+
+  # Export to shelx
+  dials.export scaled.expt scaled.refl format=shelx
+  dials.export scaled.expt scaled.refl format=shelx shelx.hklout=dials.hkl
+  dials.export scaled.expt scaled.refl format=shelx composition=C3H7NO2S
 """
 
 phil_scope = parse(
@@ -211,6 +220,14 @@ phil_scope = parse(
               "mmcif file should comply with. v5_next adds support for"
               "recording unmerged data as well as additional scan metadata"
               "and statistics, however writing can be slow for large datasets."
+    scale = True
+      .type = bool
+      .help = "If True, apply a scale such that the minimum intensity is greater"
+              "than (less negative than) the mmcif.min_scale value below."
+    min_scale = -999999.0
+      .type = float
+      .help = "If mmcif.scale is True, scale all negative intensities such that"
+              "they are less negative than this value."
   }
 
   mosflm {
@@ -250,6 +267,9 @@ phil_scope = parse(
     ins = dials.ins
       .type = path
       .help = "The output ins file"
+    composition = CH
+      .type = str
+      .help = "The chemical composition of the asymmetric unit"
     scale = True
       .type = bool
       .help = "Scale reflections to maximise output precision in SHELX 8.2f format"
@@ -363,7 +383,7 @@ def export_mtz(params, experiments, reflections):
                 "Data appear to be unscaled, setting mtz.hklout = 'integrated.mtz'"
             )
 
-    m = export_mtz(
+    export_mtz(
         reflection_table,
         experiments,
         intensity_choice=params.intensity,
@@ -380,11 +400,6 @@ def export_mtz(params, experiments, reflections):
         wavelength_tolerance=params.mtz.wavelength_tolerance,
     )
 
-    summary = StringIO()
-    m.show_summary(out=summary)
-    logger.info("")
-    logger.info(summary.getvalue())
-
 
 def export_sadabs(params, experiments, reflections):
     """
