@@ -7,9 +7,11 @@ import math
 
 import numpy as np
 
+import libtbx.phil
 from cctbx import uctbx
 
 import dials.util.log
+from dials.algorithms.merging.merge import MergingStatisticsData
 from dials.algorithms.scaling.model.model import plot_scaling_models
 from dials.algorithms.scaling.observers import make_merging_stats_plots
 from dials.algorithms.scaling.scaling_library import (
@@ -44,8 +46,6 @@ Examples::
   dials.report integrated.refl integrated.expt
 """
 
-import libtbx.phil
-
 # Create the phil parameters
 phil_scope = libtbx.phil.parse(
     """
@@ -172,7 +172,6 @@ class ScanVaryingCrystalAnalyser:
         # cell plot
         dat = []
         for iexp, exp in enumerate(experiments):
-
             crystal = exp.crystal
             scan = exp.scan
 
@@ -329,7 +328,6 @@ the refinement algorithm accounting for unmodelled features in the data.
         # orientation plot
         dat = []
         for iexp, exp in enumerate(experiments):
-
             crystal = exp.crystal
             scan = exp.scan
 
@@ -1132,7 +1130,6 @@ class CentroidAnalyser:
         d["residuals_xy"]["layout"]["title"] = "Centroid residuals in X and Y"
 
         if not is_stills:
-
             d["residuals_zy"] = {
                 "data": [
                     {
@@ -1212,7 +1209,6 @@ class IntensityAnalyser:
     """Analyse the intensities."""
 
     def __init__(self, grid_size=None, pixels_per_bin=10):
-
         self.grid_size = grid_size
         self.pixels_per_bin = pixels_per_bin
 
@@ -2081,9 +2077,6 @@ class ReferenceProfileAnalyser:
         }
 
 
-from dials.algorithms.merging.merge import MergingStatisticsData
-
-
 def merging_stats_data(reflections, experiments):
     reflections["intensity"] = reflections["intensity.scale.value"]
     reflections["variance"] = reflections["intensity.scale.variance"]
@@ -2196,7 +2189,7 @@ class Analyser:
                             scaling_tables,
                         ) = merging_stats_data(rlist, experiments)
                     except DialsMergingStatisticsError as e:
-                        print(e)
+                        print(f"Error merging stats data: {e}")
                     else:
                         json_data["resolution_graphs"] = resolution_plots
                         json_data["xtriage_output"] = xtriage_output
@@ -2207,7 +2200,6 @@ class Analyser:
                         json_data["image_range_tables"] = image_range_tables
 
         if self.params.output.html is not None:
-
             from jinja2 import ChoiceLoader, Environment, PackageLoader
 
             loader = ChoiceLoader(
