@@ -40,12 +40,12 @@ from __future__ import annotations
 
 import logging
 import sys
-from io import StringIO
 
 from libtbx import phil
 
 from dials.algorithms.scaling.algorithm import ScaleAndFilterAlgorithm, ScalingAlgorithm
 from dials.util import Sorry, log, show_mail_handle_errors
+from dials.util.export_mtz import log_summary
 from dials.util.options import ArgumentParser, reflections_and_experiments_from_files
 from dials.util.version import dials_version
 
@@ -135,10 +135,8 @@ def _export_merged_mtz(params, experiments, joint_table):
     mtz_file = merge_data_to_mtz(merge_params, experiments, [joint_table])
     logger.disabled = False
     logger.info("\nWriting merged data to %s", (params.output.merged_mtz))
-    out = StringIO()
-    mtz_file.show_summary(out=out)
-    logger.info(out.getvalue())
-    mtz_file.write(params.output.merged_mtz)
+    log_summary(mtz_file)
+    mtz_file.write_to_file(params.output.merged_mtz)
 
 
 def _export_unmerged_mtz(params, experiments, reflection_table):
