@@ -7,6 +7,7 @@ import numpy as np
 
 import iotbx.phil
 from cctbx import uctbx
+from dxtbx.model import ExperimentType
 from dxtbx.model.experiment_list import ExperimentListFactory
 from scitbx.math import five_number_summary
 
@@ -87,11 +88,14 @@ def beam_centre_raw_image_px(detector, s0):
     return x_px + offset[0], y_px + offset[1]
 
 
-def show_beam(detector, beam):
-
+def show_beam(detector, beam, experiment_type: ExperimentType | None = None):
     # standard static beam model string
     s = str(beam)
 
+    # time of flight experiments have no scan points
+    if experiment_type == ExperimentType.TOF:
+        return s
+
     # report whether the beam is scan-varying
     if beam.num_scan_points > 0:
         s += "    s0 sampled at " + str(beam.num_scan_points) + " scan points\n"
@@ -167,7 +171,6 @@ def show_beam(detector, beam):
 
 
 def show_goniometer(goniometer):
-
     # standard static goniometer model string
     s = str(goniometer)
 
@@ -247,7 +250,6 @@ def run(args=None):
 
 
 def show_experiments(experiments, show_scan_varying=False):
-
     text = []
     for i_expt, expt in enumerate(experiments):
         text.append("Experiment %i:" % i_expt)
@@ -263,16 +265,20 @@ def show_experiments(experiments, show_scan_varying=False):
         except AttributeError:
             pass
         text.append(str(expt.detector))
+        if expt.get_type() == ExperimentType.TOF:
+            min_wavelength = min(expt.beam.get_wavelength_range())
+            s0 = tuple([i / min_wavelength for i in expt.beam.get_unit_s0()])
+        else:
+            s0 = expt.beam.get_s0()
         text.append(
-            "Max resolution (at corners): %f"
-            % (expt.detector.get_max_resolution(expt.beam.get_s0()))
+            "Max resolution (at corners): %f" % (expt.detector.get_max_resolution(s0))
         )
         text.append(
             "Max resolution (inscribed):  %f"
-            % (expt.detector.get_max_inscribed_resolution(expt.beam.get_s0()))
+            % (expt.detector.get_max_inscribed_resolution(s0))
         )
         text.append("")
-        text.append(show_beam(expt.detector, expt.beam))
+        text.append(show_beam(expt.detector, expt.beam, expt.get_type()))
         if expt.scan is not None:
             text.append(str(expt.scan))
         if expt.goniometer is not None:
@@ -307,7 +313,6 @@ def show_experiments(experiments, show_scan_varying=False):
 
 
 def show_image_statistics(experiments, im_type):
-
     if im_type == "raw":
         raw = True
     elif im_type == "corrected":
@@ -315,7 +320,7 @@ def show_image_statistics(experiments, im_type):
     else:
         raise ValueError(f"Unknown im_type: {im_type}")
 
-    # To show image statistics, check_format has to be true. So we have to reinstatiate
+    # To show image statistics, check_format has to be true. So we have to reinstantiate
     # the experiment list here
     try:
         experiments = ExperimentListFactory.from_json(
@@ -430,7 +435,6 @@ def show_reflections(
     max_reflections=None,
     show_identifiers=False,
 ):
-
     text = []
 
     from orderedset import OrderedSet
@@ -650,7 +654,6 @@ def show_reflections(
         column = flex.std_string()
         max_element_lengths = [c.max_element_length() for c in c_strings]
         for i in range(len(c_strings[0])):
-
             column.append(
                 f"%{len(key)}s"
                 % ", ".join(
