@@ -277,7 +277,7 @@ def _dials_phil_str():
       transformation = 6
         .type = int(value_min=0, value_max=6)
         .multiple = False
-        .help = The index number(s) of the modulus=2 sublattice transformation(s) used to produce distince coset results. \
+        .help = The index number(s) of the modulus=2 sublattice transformation(s) used to produce distance coset results. \
                 0=Double a, 1=Double b, 2=Double c, 3=C-face centering, 4=B-face centering, 5=A-face centering, 6=Body centering \
                 See Sauter and Zwart, Acta D (2009) 65:553
     }
@@ -561,7 +561,6 @@ class Script:
             log.config(verbosity=options.verbose, logfile=logfile)
 
         else:
-
             # Configure logging
             log.config(verbosity=options.verbose, logfile="dials.process.log")
 
@@ -1017,7 +1016,6 @@ class Processor:
         debug_file_handle.close()
 
     def process_experiments(self, tag, experiments):
-
         if not self.params.output.composite_output:
             self.setup_filenames(tag)
         self.tag = tag
@@ -1378,7 +1376,6 @@ The detector is reporting a gain of %f but you have also supplied a gain of %f.
         else:
             # Dump experiments to disk
             if self.params.output.refined_experiments_filename:
-
                 experiments.as_json(self.params.output.refined_experiments_filename)
 
             if self.params.output.indexed_filename:
@@ -1543,7 +1540,6 @@ The detector is reporting a gain of %f but you have also supplied a gain of %f.
         else:
             # Dump experiments to disk
             if self.params.output.integrated_experiments_filename:
-
                 experiments.as_json(self.params.output.integrated_experiments_filename)
 
             if self.params.output.integrated_filename:
@@ -1810,19 +1806,11 @@ The detector is reporting a gain of %f but you have also supplied a gain of %f.
                         dest=destrank,
                     )
 
-                    self.all_imported_experiments = (
-                        self.all_strong_reflections
-                    ) = (
+                    self.all_imported_experiments = self.all_strong_reflections = (
                         self.all_indexed_experiments
-                    ) = (
-                        self.all_indexed_reflections
-                    ) = (
+                    ) = self.all_indexed_reflections = (
                         self.all_integrated_experiments
-                    ) = (
-                        self.all_integrated_reflections
-                    ) = (
-                        self.all_coset_experiments
-                    ) = (
+                    ) = self.all_integrated_reflections = self.all_coset_experiments = (
                         self.all_coset_reflections
                     ) = self.all_int_pickles = self.all_integrated_reflections = []
 
@@ -1831,7 +1819,6 @@ The detector is reporting a gain of %f but you have also supplied a gain of %f.
                 len(self.all_imported_experiments) > 0
                 and self.params.output.experiments_filename
             ):
-
                 self.all_imported_experiments.as_json(
                     self.params.output.experiments_filename
                 )
@@ -1848,7 +1835,6 @@ The detector is reporting a gain of %f but you have also supplied a gain of %f.
                 len(self.all_indexed_experiments) > 0
                 and self.params.output.refined_experiments_filename
             ):
-
                 self.all_indexed_experiments.as_json(
                     self.params.output.refined_experiments_filename
                 )
@@ -1865,7 +1851,6 @@ The detector is reporting a gain of %f but you have also supplied a gain of %f.
                 len(self.all_integrated_experiments) > 0
                 and self.params.output.integrated_experiments_filename
             ):
-
                 self.all_integrated_experiments.as_json(
                     self.params.output.integrated_experiments_filename
                 )
@@ -1884,7 +1869,6 @@ The detector is reporting a gain of %f but you have also supplied a gain of %f.
                     len(self.all_coset_experiments) > 0
                     and self.params.output.coset_experiments_filename
                 ):
-
                     self.all_coset_experiments.as_json(
                         self.params.output.coset_experiments_filename
                     )
