@@ -9,7 +9,7 @@ import random
 import sys
 
 import iotbx.phil
-from cctbx import sgtbx, uctbx
+from cctbx import sgtbx
 from cctbx.sgtbx.bravais_types import bravais_lattice
 from cctbx.sgtbx.lattice_symmetry import metric_subgroups
 from dxtbx.model import ExperimentList
@@ -17,7 +17,10 @@ from libtbx import Auto
 
 import dials.util
 from dials.algorithms.merging.merge import prepare_merged_reflection_table
-from dials.algorithms.symmetry import resolution_filter_from_reflections_experiments
+from dials.algorithms.symmetry import (
+    median_unit_cell,
+    resolution_filter_from_reflections_experiments,
+)
 from dials.algorithms.symmetry.absences.laue_groups_info import (
     laue_groups as laue_groups_for_absence_analysis,
 )
@@ -102,9 +105,9 @@ systematic_absences {
     .help = "Use fourier analysis or direct analysis of I/sigma to determine"
             "likelihood of systematic absences"
 
-  significance_level = *0.95 0.975 0.99
-    .type = choice
-    .help = "Signficance to use when testing whether axial reflections are "
+  significance_level = 0.95
+    .type = float(value_min=0, value_max=1)
+    .help = "Significance to use when testing whether axial reflections are "
             "different to zero (absences and reflections in reflecting condition)."
 
 }
@@ -127,14 +130,6 @@ output {
 )
 
 
-def median_unit_cell(experiments):
-    uc_params = [flex.double() for i in range(6)]
-    for c in experiments.crystals():
-        for i, p in enumerate(c.get_unit_cell().parameters()):
-            uc_params[i].append(p)
-    return uctbx.unit_cell(parameters=[flex.median(p) for p in uc_params])
-
-
 def unit_cells_are_similar_to(
     experiments, unit_cell, relative_length_tolerance, absolute_angle_tolerance
 ):
@@ -163,7 +158,7 @@ def change_of_basis_ops_to_minimum_cell(
 
     Returns: The experiments and reflections mapped to the minimum cell
     """
-
+    logger.info("Mapping all input cells to a common minimum cell")
     median_cell = median_unit_cell(experiments)
     unit_cells_are_similar = unit_cells_are_similar_to(
         experiments, median_cell, relative_length_tolerance, absolute_angle_tolerance
@@ -184,6 +179,19 @@ def change_of_basis_ops_to_minimum_cell(
         cb_op_best_to_min = group["best_subsym"].change_of_basis_op_to_minimum_cell()
         cb_ops = [cb_op_best_to_min * group["cb_op_inp_best"]] * len(experiments)
     else:
+        if len(set(centring_symbols)) > 1:
+            logger.info(
+                f"Multiple lattice centerings in input cells: {', '.join(list(set(centring_symbols)))}.\n"
+                + "Attempting to map to a common minimum cell through the most common lattice group."
+            )
+        else:
+            median_params = ", ".join(f"{i:.2f}" for i in median_cell.parameters())
+            logger.info(
+                f"Some input cells are not sufficiently similar to the median cell:\n  {median_params}\n"
+                + f"  within a relative_length_tolerance of {relative_length_tolerance}\n"
+                + f"  and an absolute_angle_tolerance of {absolute_angle_tolerance}.\n"
+                + "Attempting to map to a common minimum cell through the most common lattice group."
+            )
         groups = [
             metric_subgroups(
                 expt.crystal.get_crystal_symmetry(),
@@ -198,6 +206,7 @@ def change_of_basis_ops_to_minimum_cell(
         )
         target_group = counter.most_common()[0][0]
         cb_ops = []
+        best_cells = []
         for expt in experiments:
             groups = metric_subgroups(
                 expt.crystal.get_crystal_symmetry(),
@@ -211,6 +220,7 @@ def change_of_basis_ops_to_minimum_cell(
                     group = g
             if group:
                 cb_ops.append(group["cb_op_inp_best"])
+                best_cells.append(group["best_subsym"].unit_cell())
             else:
                 cb_ops.append(None)
                 logger.info(
@@ -218,6 +228,43 @@ def change_of_basis_ops_to_minimum_cell(
                     f"{expt.crystal.get_crystal_symmetry()}\n"
                     f"Target symmetry: {target_group.info()}"
                 )
+        # now get median best cell
+        from cctbx import uctbx
+
+        uc_params = [flex.double() for i in range(6)]
+        for unit_cell in best_cells:
+            for i, p in enumerate(unit_cell.parameters()):
+                uc_params[i].append(p)
+        overall_best_unit_cell = uctbx.unit_cell(
+            parameters=[flex.median(p) for p in uc_params]
+        )
+        n = 0
+        for i, cb_op in enumerate(cb_ops):
+            if cb_op is not None:
+                best_cell = best_cells[n]
+                n += 1
+                if not best_cell.is_similar_to(
+                    overall_best_unit_cell,
+                    relative_length_tolerance,
+                    absolute_angle_tolerance,
+                ):
+                    best_params = ", ".join(
+                        f"{i:.2f}" for i in overall_best_unit_cell.parameters()
+                    )
+                    this_params = ", ".join(f"{i:.2f}" for i in best_cell.parameters())
+                    logger.info(
+                        f"Unable to map input cell for dataset {i} to a minimum cell through\n"
+                        + f"a consistent best cell in space group {target_group.info()}.\n"
+                        + f"Incompatible best cells:\n  {best_params},\n  {this_params},\n"
+                        + f"  within a relative_length_tolerance of {relative_length_tolerance}\n"
+                        + f"  and an absolute_angle_tolerance of {absolute_angle_tolerance}"
+                    )
+                    cb_ops[i] = None
+        if not any(cb_ops):
+            raise ValueError(
+                "Exiting symmetry analysis: Unable to map any cells to a minimum cell through a consistent best cell"
+            )
+
         ref_expts = ExperimentList(
             [expt for expt, cb_op in zip(experiments, cb_ops) if cb_op]
         ).change_basis(list(filter(None, cb_ops)))
@@ -329,6 +376,14 @@ def symmetry(experiments, reflection_tables, params=None):
             params.relative_length_tolerance,
             params.absolute_angle_tolerance,
         )
+        if None in cb_ops:
+            indices = [str(i + 1) for i, v in enumerate(cb_ops) if v is None]
+            raise ValueError(
+                "Exiting symmetry analysis: Unable to match all cells to target symmetry.\n"
+                + "This may be avoidable by increasing the absolute_angle_tolerance or relative_length_tolerance,\n"
+                + "if the cells are similar enough.\n"
+                + f"Alternatively, remove dataset number{'s' if len(indices) > 1 else ''} {', '.join(indices)} from the input"
+            )
         reflection_tables = eliminate_sys_absent(experiments, reflection_tables)
         experiments, reflection_tables = apply_change_of_basis_ops(
             experiments, reflection_tables, cb_ops
