@@ -6,6 +6,8 @@ import wx
 from wx.lib.agw.floatspin import EVT_FLOATSPIN, FloatSpin
 
 import wxtbx
+from scitbx import matrix
+from scitbx.array_family import flex
 from wxtbx import metallicbutton
 from wxtbx.phil_controls import EVT_PHIL_CONTROL
 from wxtbx.phil_controls.floatctrl import FloatCtrl as _FloatCtrl
@@ -13,10 +15,10 @@ from wxtbx.phil_controls.intctrl import IntCtrl
 from wxtbx.phil_controls.strctrl import StrCtrl
 
 import dials.util.masking
+from dials.algorithms.polygon import clip
 
 
 class FloatCtrl(_FloatCtrl):
-
     # override OnFocusLostMethod since calling event.Skip() causes bad things to
     # happen (for reasons I don't understand)
     def OnFocusLost(self, event):
@@ -571,15 +573,12 @@ class MaskSettingsPanel(wx.Panel):
         )
 
         if self._resolution_range_d_min > 0 and self._resolution_range_d_max > 0:
-
             self.params.masking.resolution_range.append(
                 (self._resolution_range_d_min, self._resolution_range_d_max)
             )
             self._resolution_range_d_min = 0
             self._resolution_range_d_max = 0
 
-        from dials.util import masking
-
         untrusted_rectangle = self.untrusted_rectangle_ctrl.GetValue().strip()
         if len(untrusted_rectangle.strip()) > 0:
             rectangle = untrusted_rectangle.strip().replace(",", " ").split(" ")
@@ -590,7 +589,7 @@ class MaskSettingsPanel(wx.Panel):
             except Exception:
                 pass
             else:
-                untrusted = masking.phil_scope.extract().untrusted[0]
+                untrusted = dials.util.masking.phil_scope.extract().untrusted[0]
                 untrusted.panel = panel
                 untrusted.rectangle = rectangle
                 self.params.masking.untrusted.append(untrusted)
@@ -606,7 +605,7 @@ class MaskSettingsPanel(wx.Panel):
             except Exception:
                 pass
             else:
-                untrusted = masking.phil_scope.extract().untrusted[0]
+                untrusted = dials.util.masking.phil_scope.extract().untrusted[0]
                 untrusted.panel = panel
                 untrusted.polygon = polygon
                 self.params.masking.untrusted.append(untrusted)
@@ -621,7 +620,7 @@ class MaskSettingsPanel(wx.Panel):
             except Exception:
                 pass
             else:
-                untrusted = masking.phil_scope.extract().untrusted[0]
+                untrusted = dials.util.masking.phil_scope.extract().untrusted[0]
                 untrusted.panel = panel
                 untrusted.circle = circle
                 self.params.masking.untrusted.append(untrusted)
@@ -674,6 +673,8 @@ class MaskSettingsPanel(wx.Panel):
         )
         image_viewer_frame.mask_image_viewer = mask
         image_viewer_frame.update_settings(layout=False)
+        if image_viewer_frame.settings.show_mask:
+            image_viewer_frame.reload_image()
 
     def OnLeftDown(self, event):
         if not event.ShiftDown():
@@ -687,8 +688,11 @@ class MaskSettingsPanel(wx.Panel):
                 self._circle_radius = None
                 return
             elif self._mode_polygon:
-                self._mode_polygon_points.append(click_posn)
+                xgeo, ygeo = self._pyslip.ConvertView2Geo(click_posn)
+
+                self._mode_polygon_points.append((xgeo, ygeo))
                 self.DrawPolygon(self._mode_polygon_points)
+
         event.Skip()
 
     def OnLeftUp(self, event):
@@ -699,12 +703,16 @@ class MaskSettingsPanel(wx.Panel):
                 self._rectangle_x1y1 = click_posn
                 x0, y0 = self._rectangle_x0y0
                 x1, y1 = self._rectangle_x1y1
-                self.AddUntrustedRectangle(x0, y0, x1, y1)
-                self._pyslip.DeleteLayer(self._mode_rectangle_layer)
-                self._mode_rectangle_layer = None
-                self.mode_rectangle_button.SetValue(False)
-                self.OnUpdate(event)
-                return
+                try:
+                    self.AddUntrustedRectangle(x0, y0, x1, y1)
+                except Exception as e:
+                    wx.MessageBox(str(e))
+                finally:
+                    self._pyslip.DeleteLayer(self._mode_rectangle_layer)
+                    self._mode_rectangle_layer = None
+                    self.mode_rectangle_button.SetValue(False)
+                    self.OnUpdate(event)
+                    return
 
             elif self._mode_circle and self._circle_xy is not None:
                 xc, yc = self._circle_xy
@@ -713,7 +721,7 @@ class MaskSettingsPanel(wx.Panel):
                 try:
                     self.AddUntrustedCircle(xc, yc, xedge, yedge)
                 except Exception as e:
-                    print(e)
+                    wx.MessageBox(str(e))
                 finally:
                     self._pyslip.DeleteLayer(self._mode_circle_layer)
                     self._mode_circle_layer = None
@@ -769,8 +777,6 @@ class MaskSettingsPanel(wx.Panel):
         xc, yc = self._pyslip.ConvertView2Geo((xc, yc))
         xedge, yedge = self._pyslip.ConvertView2Geo((xedge, yedge))
 
-        from scitbx import matrix
-
         center = matrix.col((xc, yc))
         edge = matrix.col((xedge, yedge))
         r = (center - edge).length()
@@ -798,7 +804,6 @@ class MaskSettingsPanel(wx.Panel):
         )
 
     def DrawPolygon(self, vertices):
-
         if self._mode_polygon_layer:
             self._pyslip.DeleteLayer(self._mode_polygon_layer)
             self._mode_polygon_layer = None
@@ -809,10 +814,7 @@ class MaskSettingsPanel(wx.Panel):
         for i in range(len(vertices) - 1):
             polygon_data.append(
                 (
-                    (
-                        self._pyslip.ConvertView2Geo(vertices[i]),
-                        self._pyslip.ConvertView2Geo(vertices[i + 1]),
-                    ),
+                    (vertices[i], vertices[i + 1]),
                     d,
                 )
             )
@@ -828,37 +830,120 @@ class MaskSettingsPanel(wx.Panel):
             )
 
     def AddUntrustedPolygon(self, vertices):
-        if len(vertices) < 4:
+        if len(vertices) < 3:
             return
+
+        # flex_image works in (slow, fast) coordinates, while others are in (fast, slow).
         vertices.append(vertices[0])
-        vertices = [self._pyslip.ConvertView2Geo(v) for v in vertices]
         vertices = [
             self._pyslip.tiles.map_relative_to_picture_fast_slow(*v) for v in vertices
         ]
+        flex_vertices = flex.vec2_double(vertices)
 
-        point_ = []
-        panel_id = None
-        for p in vertices:
-            p1, p0, p_id = self._pyslip.tiles.flex_image.picture_to_readout(p[1], p[0])
-            assert p_id >= 0, "Point must be within a panel"
-            if panel_id is not None:
-                assert (
-                    panel_id == p_id
-                ), "All points must be contained within a single panel"
-            panel_id = p_id
-            point_.append((p0, p1))
-        vertices = point_
+        numerical_fudges = [
+            (0, 0),
+            (-1, 0),
+            (+1, 0),
+            (0, -1),
+            (0, +1),
+            (+1, +1),
+            (+1, -1),
+            (-1, +1),
+            (-1, -1),
+        ]
 
-        from libtbx.utils import flat_list
+        for i, panel in enumerate(self._pyslip.tiles.raw_image.get_detector()):
+            # Get sensor bounding boxes in the projected coordinate system
+            panel_size = panel.get_image_size()
+            panel_corners = [
+                (0, 0),
+                (panel_size[0], 0),
+                (panel_size[0], panel_size[1]),
+                (0, panel_size[1]),
+            ]
 
-        from dials.util import masking
+            panel_corners_picture = []
+            for corner in panel_corners:
+                p = self._pyslip.tiles.flex_image.tile_readout_to_picture(
+                    i, corner[1], corner[0]
+                )
+                panel_corners_picture.append((p[1], p[0]))
+            panel_corners_picture = flex.vec2_double(panel_corners_picture)
 
-        region = masking.phil_scope.extract().untrusted[0]
-        points = flat_list(vertices)
-        region.polygon = [int(p) for p in points]
-        region.panel = panel_id
+            # and intersect with the user-drawn polygon.
+            clipped = clip.simple_with_convex(flex_vertices, panel_corners_picture)
+            # The order matters for clipping; so we try the other if the first order failed.
+            if len(clipped) == 0:
+                clipped = clip.simple_with_convex(
+                    flex_vertices, panel_corners_picture.reversed()
+                )
+            if len(clipped) > 0:
+                # print("Input vertices", vertices)
+                # print("Intersection with panel", i, list(clipped))
+                # print("Panel edges", list(panel_corners_picture))
 
-        self.params.masking.untrusted.append(region)
+                points = []
+                for p in clipped:
+                    if p in flex_vertices:
+                        # p is a vertex provided by a user; treat as is.
+                        p1_best, p0_best, p_id = (
+                            self._pyslip.tiles.flex_image.picture_to_readout(p[1], p[0])
+                        )
+                        assert p_id == i
+                    else:
+                        # p is a vertex generated by the clipping procedure.
+                        # Thus, p must be on an edge of a panel.
+                        # Because of numerical errors in converting between the projected coordinate system
+                        # and the panel coordinate system, we have to allow some errors.
+                        # Otherwise the point might be outside a panel.
+                        # Moving around one pixel is enough to bring the point inside (on an edge of) a panel.
+
+                        n_touching_edges_best = 0
+                        for f in numerical_fudges:
+                            p1, p0, p_id = (
+                                self._pyslip.tiles.flex_image.picture_to_readout(
+                                    p[1] + f[1], p[0] + f[0]
+                                )
+                            )
+                            if p_id != i:
+                                continue
+                            p1, p0 = round(p1), round(p0)
+
+                            # In the polygon masking code, an integer coordinate means a corner of a pixel,
+                            # while a pixel's center (0.5, 0.5) must be within the polygon for the pixel to be masked.
+                            # Thus, to fully mask a panel edge, we must add one.
+                            n_touching_edges = 0
+                            if p0 == 0:
+                                n_touching_edges += 1
+                            elif p0 == panel_size[0] - 1:
+                                n_touching_edges += 1
+                                p0 = panel_size[0]
+
+                            if p1 == 0:
+                                n_touching_edges += 1
+                            elif p1 == panel_size[1] - 1:
+                                n_touching_edges += 1
+                                p1 = panel_size[1]
+
+                            if n_touching_edges > n_touching_edges_best:
+                                p1_best, p0_best = p1, p0
+                                n_touching_edges_best = n_touching_edges
+                        assert n_touching_edges_best > 0
+
+                    # print("Added vertex", p_id, p1_best, p0_best)
+                    points.append((p0_best, p1_best))
+
+                # polygon masking does not allow triangles so make it a quadrilateral.
+                if len(points) == 3:
+                    points.append(points[0])
+
+                from libtbx.utils import flat_list
+
+                region = dials.util.masking.phil_scope.extract().untrusted[0]
+                region.polygon = [int(p) for p in flat_list(points)]
+                region.panel = i
+
+                self.params.masking.untrusted.append(region)
 
     def AddUntrustedRectangle(self, x0, y0, x1, y1):
         x0, y0 = self._pyslip.ConvertView2Geo((x0, y0))
@@ -907,16 +992,13 @@ class MaskSettingsPanel(wx.Panel):
         x1 = min(panel.get_image_size()[0], x1)
         y1 = min(panel.get_image_size()[1], y1)
 
-        from dials.util import masking
-
-        region = masking.phil_scope.extract().untrusted[0]
+        region = dials.util.masking.phil_scope.extract().untrusted[0]
         region.rectangle = [int(x0), int(x1), int(y0), int(y1)]
         region.panel = panel_id
 
         self.params.masking.untrusted.append(region)
 
     def AddUntrustedCircle(self, xc, yc, xedge, yedge):
-
         points = [(xc, yc), (xedge, yedge)]
 
         points = [self._pyslip.ConvertView2Geo(p) for p in points]
@@ -939,17 +1021,13 @@ class MaskSettingsPanel(wx.Panel):
 
         (xc, yc), (xedge, yedge) = points
 
-        from scitbx import matrix
-
         center = matrix.col((xc, yc))
         edge = matrix.col((xedge, yedge))
         r = (center - edge).length()
         if r == 0:
             return
 
-        from dials.util import masking
-
-        region = masking.phil_scope.extract().untrusted[0]
+        region = dials.util.masking.phil_scope.extract().untrusted[0]
         region.circle = [int(xc), int(yc), int(r)]
         region.panel = panel_id
 
