@@ -362,7 +362,7 @@ class UCSettingsPanel(wx.Panel):
             pass
         dc.SetPen(wx.Pen(colour))
         dc.SetBrush(wx.Brush(colour, wx.TRANSPARENT))
-        for (lon, lat, place, radius, colour, x_off, y_off, pdata) in data:
+        for lon, lat, place, radius, colour, x_off, y_off, pdata in data:
             (x, y) = self._pyslip.ConvertGeo2View((lon, lat))
             dc.DrawCircle(int(x), int(y), int(radius * scale))
 
