@@ -8,6 +8,7 @@ import enum
 import logging
 import math
 import typing
+import warnings
 
 import numpy as np
 import scipy.optimize
@@ -91,8 +92,9 @@ def tanh_fit(x, y, degree=None, n_obs=None):
     p0 = np.array([0.2, 0.4])  # starting parameter estimates
     sigma = np.array(standard_errors)
     x = np.array(x)
-
-    result = scipy.optimize.curve_fit(tanh_cchalf, x, y, p0, sigma=sigma)
+    with warnings.catch_warnings():
+        warnings.simplefilter("ignore", scipy.optimize.OptimizeWarning)
+        result = scipy.optimize.curve_fit(tanh_cchalf, x, y, p0, sigma=sigma)
 
     r = result[0][0]
     s0 = result[0][1]
@@ -108,9 +110,11 @@ def log_fit(x, y, degree=5, n_obs=None):
 
     x, y should be iterables containing floats of the same size. The order is the order
     of polynomial to use for this fit. This will be useful for e.g. I/sigma."""
-
+    sel = y > 0
+    y_sel = y.select(sel)
+    x_sel = x.select(sel)
     fit = curve_fitting.univariate_polynomial_fit(
-        x, flex.log(y), degree=degree, max_iterations=100
+        x_sel, flex.log(y_sel), degree=degree, max_iterations=100
     )
     f = curve_fitting.univariate_polynomial(*fit.params)
     return flex.exp(f(x))
@@ -121,9 +125,11 @@ def log_inv_fit(x, y, degree=5, n_obs=None):
 
     x, y should be iterables, the order of the polynomial for the transformed
     fit needs to be specified. This will be useful for e.g. Rmerge."""
-
+    sel = y > 0
+    y_sel = y.select(sel)
+    x_sel = x.select(sel)
     fit = curve_fitting.univariate_polynomial_fit(
-        x, flex.log(1 / y), degree=degree, max_iterations=100
+        x_sel, flex.log(1 / y_sel), degree=degree, max_iterations=100
     )
     f = curve_fitting.univariate_polynomial(*fit.params)
     return 1 / flex.exp(f(x))
@@ -236,9 +242,11 @@ def _get_cc_half_critical_values(merging_stats, cc_half_method):
         ).reversed()
     elif merging_stats.overall.cc_one_half_critical_value is not None:
         critical = [
-            b.cc_one_half_critical_value
-            if b.cc_one_half_critical_value is not None
-            else 0.0
+            (
+                b.cc_one_half_critical_value
+                if b.cc_one_half_critical_value is not None
+                else 0.0
+            )
             for b in merging_stats.bins
         ]
         return flex.double(critical).reversed()
@@ -509,7 +517,6 @@ class Resolutionizer:
     """A class to calculate things from merging reflections."""
 
     def __init__(self, i_obs, params, batches=None, reference=None):
-
         self._params = params
         self._reference = reference
 
@@ -629,9 +636,9 @@ class Resolutionizer:
                 self._merging_statistics,
                 limit,
                 cc_half_method=self._params.cc_half_method,
-                model=tanh_fit
-                if self._params.cc_half_fit == "tanh"
-                else polynomial_fit,
+                model=(
+                    tanh_fit if self._params.cc_half_fit == "tanh" else polynomial_fit
+                ),
             )
         elif metric == metrics.CC_REF:
             return self._resolution_cc_ref(limit=self._params.cc_ref)
