@@ -63,9 +63,7 @@ def test_spots_xds(tmp_path):
  1317.52 1171.59 19.28 120.00  6 -4 6
  1260.25 1300.55 13.67 116.00  -4 2 6
  1090.27 1199.47 41.49 114.00  -2 3 -13
-""".split(
-            "\n"
-        )
+""".split("\n")
     ]
 
 
