@@ -8,7 +8,7 @@ import subprocess
 import pytest
 
 import scitbx.matrix
-from cctbx import sgtbx, uctbx
+from cctbx import crystal, sgtbx, uctbx
 from dxtbx.model import Crystal, Experiment, ExperimentList, Scan
 from dxtbx.serialize import load
 
@@ -416,9 +416,6 @@ def test_change_of_basis_ops_to_minimum_cell_mpro():
     )
 
 
-from cctbx import crystal
-
-
 def test_change_of_basis_ops_to_minimum_cell_with_outlier():
     symmetries = [
         crystal.symmetry(unit_cell=uc, space_group="P1")
